@@ -119,7 +119,7 @@ class Progress:
                     ).select(table.id, Sum(progress.progress),
                     where=where,
                     group_by=table.id))
-            invoiced_progress.update(dict(cursor.fetchall()))
+            invoiced_progress.update(dict(cursor))
 
             for work in sub_works:
                 delta = (
@@ -169,7 +169,7 @@ class Progress:
                     Sum(progress.progress * invoice_line.unit_price),
                     where=where,
                     group_by=table.id))
-            for work_id, amount in cursor.fetchall():
+            for work_id, amount in cursor:
                 if not isinstance(amount, Decimal):
                     amount = Decimal(str(amount))
                 work = ids2work[work_id]
@@ -181,7 +181,7 @@ class Progress:
                     condition=table.company == company.id
                     ).select(table.id, company.currency,
                     where=where))
-            work2currency.update(cursor.fetchall())
+            work2currency.update(cursor)
 
         currencies = Currency.browse(set(work2currency.values()))
         id2currency = {c.id: c for c in currencies}
@@ -207,6 +207,17 @@ class Progress:
 
 class Timesheet:
 
+    project_invoice_timesheet_up_to = fields.Date(
+        "Invoice up to",
+        states={
+            'invisible': Eval('project_invoice_method') != 'timesheet',
+            },
+        depends=['project_invoice_method'],
+        help="Limits which timesheet lines get invoiced to "
+        "only those before the date.")
+    invoice_timesheet_up_to = fields.Function(fields.Date(
+            "Invoice up to"), 'on_change_with_invoice_timesheet_up_to')
+
     @classmethod
     def __setup__(cls):
         super().__setup__()
@@ -221,6 +232,14 @@ class Timesheet:
         if 'invoice_method' not in cls.product.depends:
             cls.product.depends.append('invoice_method')
 
+    @fields.depends('type', 'project_invoice_timesheet_up_to',
+        'parent', '_parent_parent.invoice_timesheet_up_to')
+    def on_change_with_invoice_timesheet_up_to(self, name=None):
+        if self.type == 'project':
+            return self.project_invoice_timesheet_up_to
+        elif self.parent:
+            return self.parent.invoice_timesheet_up_to
+
     @classmethod
     def _get_quantity_to_invoice_timesheet(cls, works):
         pool = Pool()
@@ -235,7 +254,7 @@ class Timesheet:
             cursor.execute(*line.select(line.work, Sum(line.duration),
                     where=red_sql & (line.invoice_line == Null),
                     group_by=line.work))
-            for twork_id, duration in cursor.fetchall():
+            for twork_id, duration in cursor:
                 if duration:
                     # SQLite uses float for SUM
                     if not isinstance(duration, datetime.timedelta):
@@ -285,13 +304,13 @@ class Timesheet:
                     Sum(timesheet_line.duration * invoice_line.unit_price),
                     where=where,
                     group_by=table.id))
-            amounts.update(cursor.fetchall())
+            amounts.update(cursor)
 
             cursor.execute(*table.join(company,
                     condition=table.company == company.id
                     ).select(table.id, company.currency,
                     where=where))
-            work2currency.update(cursor.fetchall())
+            work2currency.update(cursor)
 
         currencies = Currency.browse(set(work2currency.values()))
         id2currency = {c.id: c for c in currencies}
@@ -311,10 +330,11 @@ class Timesheet:
         except AttributeError:
             origins = []
         if self.invoice_method == 'timesheet':
+            up_to = self.invoice_timesheet_up_to or datetime.date.max
             origins.extend(
                 l for tw in self.timesheet_works
                 for l in tw.timesheet_lines
-                if not l.invoice_line)
+                if not l.invoice_line and l.date <= up_to)
         return origins
 
 
@@ -534,7 +554,7 @@ class Work(Effort, Progress, Timesheet, metaclass=PoolMeta):
                         product=product.rec_name))
         else:
             invoice_line.account = account_config.get_multivalue(
-                'default_category_account_revenue')
+                'default_category_account_revenue', company=invoice.company.id)
             if not invoice_line.account:
                 raise InvoicingError(
                     gettext('project_invoice.msg_missing_account_revenue',
@@ -543,6 +563,8 @@ class Work(Effort, Progress, Timesheet, metaclass=PoolMeta):
         invoice_line.unit_price = key['unit_price']
         invoice_line.quantity = quantity
         invoice_line.unit = key['unit']
+        invoice_line.currency = invoice.currency
+        invoice_line.company = invoice.company
 
         taxes = []
         pattern = invoice_line._get_tax_rule_pattern()
@@ -662,7 +684,7 @@ class OpenInvoice(Wizard):
 
     def do_open_(self, action):
         works = self.model.search([
-                ('parent', 'child_of', map(int, self.records)),
+                ('parent', 'child_of', list(map(int, self.records))),
                 ])
         invoice_ids = set()
         for work in works:
